@@ -1,8 +1,10 @@
+/* $Cambridge: exim/src/src/lookups/pgsql.c,v 1.11 2009/11/16 19:50:38 nm4 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Thanks to Petr Cech for contributing the original code for these
@@ -70,12 +72,32 @@ while ((cn = pgsql_connections) != NULL)
   pgsql_connections = cn->next;
   DEBUG(D_lookup) debug_printf("close PGSQL connection: %s\n", cn->server);
   PQfinish(cn->handle);
-  store_free(cn->server);
-  store_free(cn);
   }
 }
 
 
+/*************************************************
+*       Notice processor function for pgsql      *
+*************************************************/
+
+/* This function is passed to pgsql below, and called for any PostgreSQL
+"notices". By default they are written to stderr, which is undesirable.
+
+Arguments:
+  arg        an opaque user cookie (not used)
+  message    the notice
+
+Returns:     nothing
+*/
+
+static void
+notice_processor(void *arg, const char *message)
+{
+arg = arg;   /* Keep compiler happy */
+DEBUG(D_lookup) debug_printf("PGSQL: %s\n", message);
+}
+
+
 
 /*************************************************
 *        Internal search function                *
@@ -106,14 +128,15 @@ Arguments:
   server       the server string; this is in dynamic memory and can be updated
   resultptr    where to store the result
   errmsg       where to point an error message
-  defer_break  TRUE if no more servers are to be tried after DEFER
+  defer_break  set TRUE if no more servers are to be tried after DEFER
+  do_cache     set FALSE if data is changed
 
 Returns:       OK, FAIL, or DEFER
 */
 
 static int
 perform_pgsql_search(uschar *query, uschar *server, uschar **resultptr,
-  uschar **errmsg, BOOL *defer_break)
+  uschar **errmsg, BOOL *defer_break, BOOL *do_cache)
 {
 PGconn *pg_conn = NULL;
 PGresult *pg_result = NULL;
@@ -145,7 +168,7 @@ for (i = 2; i >= 0; i--)
     }
   *pp++ = 0;
   sdata[i] = pp;
-  if (i == 2) server_copy = string_copy_malloc(server);  /* sans password */
+  if (i == 2) server_copy = string_copy(server);  /* sans password */
   }
 
 /* The total server string has now been truncated so that what is left at the
@@ -238,17 +261,28 @@ if (cn == NULL)
 
   if(PQstatus(pg_conn) == CONNECTION_BAD)
     {
+    store_reset(server_copy);
     *errmsg = string_sprintf("PGSQL connection failed: %s",
       PQerrorMessage(pg_conn));
     PQfinish(pg_conn);
-    store_free(server_copy);
-    *defer_break = FALSE;
     goto PGSQL_EXIT;
     }
 
+  /* Set the client encoding to SQL_ASCII, which means that the server will
+  not try to interpret the query as being in any fancy encoding such as UTF-8
+  or other multibyte code that might cause problems with escaping. */
+
+  PQsetClientEncoding(pg_conn, "SQL_ASCII");
+
+  /* Set the notice processor to prevent notices from being written to stderr
+  (which is what the default does). Our function (above) just produces debug
+  output. */
+
+  PQsetNoticeProcessor(pg_conn, notice_processor, NULL);
+
   /* Add the connection to the cache */
 
-  cn = store_malloc(sizeof(pgsql_connection));
+  cn = store_get(sizeof(pgsql_connection));
   cn->server = server_copy;
   cn->handle = pg_conn;
   cn->next = pgsql_connections;
@@ -271,9 +305,12 @@ else
     case PGRES_EMPTY_QUERY:
     case PGRES_COMMAND_OK:
     /* The command was successful but did not return any data since it was
-     * not SELECT but either an INSERT, UPDATE or DELETE statement. */
+     * not SELECT but either an INSERT, UPDATE or DELETE statement. Tell the
+     * high level code to not cache this query, and clean the current cache for
+     * this handle by setting *do_cache FALSE. */
     result = string_copy(US PQcmdTuples(pg_result));
     offset = Ustrlen(result);
+    *do_cache = FALSE;
     DEBUG(D_lookup) debug_printf("PGSQL: command does not return any data "
       "but was successful. Rows affected: %s\n", result);
 
@@ -281,9 +318,15 @@ else
     break;
 
     default:
+    /* This was the original code:
     *errmsg = string_sprintf("PGSQL: query failed: %s\n",
                              PQresultErrorMessage(pg_result));
-    *defer_break = FALSE;
+    This was suggested by a user:
+    */
+
+    *errmsg = string_sprintf("PGSQL: query failed: %s (%s) (%s)\n",
+                             PQresultErrorMessage(pg_result),
+                             PQresStatus(PQresultStatus(pg_result)), query);
     goto PGSQL_EXIT;
     }
 
@@ -314,30 +357,9 @@ for (i = 0; i < num_tuples; i++)
     int j;
     for (j = 0; j < num_fields; j++)
       {
-      uschar *tmp = US PQfname(pg_result, j);
-      result = string_cat(result, &ssize, &offset, tmp, Ustrlen(tmp));
-      result = string_cat(result, &ssize, &offset, US"=", 1);
-
-      /* Quote the value if it contains spaces or is empty */
-
-      tmp = US PQgetvalue(pg_result, i, j);
-      if (*tmp == 0 || (Ustrchr(tmp, ' ') != NULL))
-	{
-	int k, t_len = Ustrlen(tmp);
-	result = string_cat(result, &ssize, &offset, US"\"", 1);
-	for(k = 0; k < t_len; k++)
-	  {
-	  if (tmp[k] == '\"' || tmp[k] == '\\')
-	     result = string_cat(result, &ssize, &offset, US"\\", 1);
-	  result = string_cat(result, &ssize, &offset, tmp+k, 1);
-	  }
-	result = string_cat(result, &ssize, &offset, US"\"", 1);
-	}
-      else
-	{
-	result = string_cat(result, &ssize, &offset, tmp, Ustrlen(tmp));
-	}
-      result = string_cat(result, &ssize, &offset, US" ", 1);
+      uschar *tmp = US PQgetvalue(pg_result, i, j);
+      result = lf_quote(US PQfname(pg_result, j), tmp, Ustrlen(tmp), result,
+        &ssize, &offset);
       }
     }
   }
@@ -351,7 +373,11 @@ if (result == NULL)
   yield = FAIL;
   *errmsg = US"PGSQL: no data found";
   }
-else result[offset] = 0;
+else
+  {
+  result[offset] = 0;
+  store_reset(result + offset + 1);
+  }
 
 /* Get here by goto from various error checks. */
 
@@ -384,32 +410,16 @@ else
 *************************************************/
 
 /* See local README for interface description. The handle and filename
-arguments are not used. Loop through a list of servers while the query is
-deferred with a retryable error. */
+arguments are not used. The code to loop through a list of servers while the
+query is deferred with a retryable error is now in a separate function that is
+shared with other SQL lookups. */
 
 int
 pgsql_find(void *handle, uschar *filename, uschar *query, int length,
-  uschar **result, uschar **errmsg)
+  uschar **result, uschar **errmsg, BOOL *do_cache)
 {
-int sep = 0;
-uschar *server;
-uschar *list = pgsql_servers;
-uschar buffer[512];
-
-DEBUG(D_lookup) debug_printf("PGSQL query: %s\n", query);
-
-while ((server = string_nextinlist(&list, &sep, buffer, sizeof(buffer)))
-        != NULL)
-  {
-  BOOL defer_break;
-  int rc = perform_pgsql_search(query, server, result, errmsg, &defer_break);
-  if (rc != DEFER || defer_break) return rc;
-  }
-
-if (pgsql_servers == NULL)
-  *errmsg = US"no PGSQL servers defined (pgsql_servers option)";
-
-return DEFER;
+return lf_sqlperform(US"PostgreSQL", US"pgsql_servers", pgsql_servers, query,
+  result, errmsg, do_cache, perform_pgsql_search);
 }
 
 
@@ -427,6 +437,16 @@ to allow escaping "on spec". If you use something like "where id="ab\%cd" it
 does treat the string as "ab%cd". So we can safely quote percent and
 underscore. [This is different to MySQL, where you can't do this.]
 
+The original code quoted single quotes as \' which is documented as valid in
+the O'Reilly book "Practical PostgreSQL" (first edition) as an alternative to
+the SQL standard '' way of representing a single quote as data. However, in
+June 2006 there was some security issue with using \' and so this has been
+changed.
+
+[Note: There is a function called PQescapeStringConn() that quotes strings.
+This cannot be used because it needs a PGconn argument (the connection handle).
+Why, I don't know. Seems odd for just string escaping...]
+
 Arguments:
   s          the string to be quoted
   opt        additional option text or NULL if none
@@ -452,7 +472,12 @@ t = quoted = store_get(Ustrlen(s) + count + 1);
 
 while ((c = *s++) != 0)
   {
-  if (Ustrchr("\n\t\r\b\'\"\\%_", c) != NULL)
+  if (c == '\'')
+    {
+    *t++ = '\'';
+    *t++ = '\'';
+    }
+  else if (Ustrchr("\n\t\r\b\"\\%_", c) != NULL)
     {
     *t++ = '\\';
     switch(c)
